var esprima = require("esprima");
var colors = require("./colors.json");
var needEndColor;

module.exports = function (code, options) {
  options = options || {};

  var ast = esprima.parse(removeShebang(code), {
    tokens: true,
    comment: true,
    range: true
  });

  var colorStartIndexes = [];
  var colorEndIndexes = [];
  ast.tokens.concat(ast.comments).forEach(function (token) {
    var range = token.range;
    colorStartIndexes[range[0]] = isOperator(token) ? "Operator" : token.type;
    colorEndIndexes[range[1]] = true;
  });

  var output = "";

  var showLineNumber = options.lineNumber;
  if (showLineNumber) {
    var lineNumber = 0;
    output += getLineNumber();
  }

  [].forEach.call(code, function (char, index) {
    if (colorEndIndexes[index]) {
      output += endColor();
    }
    var type = colorStartIndexes[index];
    if (type && colors[type]) {
      output += startColor(colors[type]);
    }
    output += char;
    if (showLineNumber && char === "\n") {
      output += getLineNumber();
    }
  })

  if(needEndColor){
    output += endColor();
  }

  return output;

  function getLineNumber() {
    return startColor(colors["LineNumber"]) + precedingBlanks(++lineNumber) + "\t" + endColor();
  }
}

function removeShebang(code) {
  return code.replace(/^#!/, "//");
}

function isOperator(token) {
  return token.type === "Punctuator" && "()[]{},.:;".indexOf(token.value) === -1;
}

function startColor(color) {
  needEndColor = true;
  var ansiColors = {
    "normal": 39,
    "black": 30,
    "red": 31,
    "green": 32,
    "yellow": 33,
    "blue": 34,
    "magenta": 35,
    "cyan": 36,
    "white": 37,
    "brightBlack": 90,
    "brightRed": 91,
    "brightGreen": 92,
    "brightYellow": 93,
    "brightBlue": 94,
    "brightMagenta": 95,
    "brightCyan": 96,
    "brightWhite": 97
  }
  return "\x1b[" + ansiColors[color] + "m";
}

function endColor() {
  needEndColor = false;
  return "\x1b[0m";
}

function precedingBlanks(str) {
  str = String(str);
  while (str.length < 6) {
    str = " " + str;
  }
  return str;
}